package org.olga.rebus.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.*;

import org.olga.rebus.structure.IRebus;

/**
 *   . 
 * 
 * @author Olga
 *
 */
public class RebusForm extends JFrame {
	private static final long serialVersionUID = 1L;
	//   
	private RebusViewPanel myView;
	//    
	private RebusResultPanel myResult;
	//,    
	private JSplitPane mySplit = new JSplitPane();
	// ""
	private JButton myButton;
	// 
	Color myRebusColor = Color.BLUE;
	// 
	Color myBackgroundColor;
	//,     
	private IRebus myRebus;
	
	/*
	 * ,    .
	 */
	void readFromFile() {
		try {
			ObjectInputStream oi = new ObjectInputStream(new FileInputStream(System.getProperty("user.home") + "/rebuses" + "/property.prt"));
			myRebusColor = (Color) oi.readObject();
			myBackgroundColor = (Color) oi.readObject();
			mySplit.setOrientation(oi.readInt());
			oi.close();
		} catch (IOException ex) {
		} catch (ClassCastException e1) {
		} catch (ClassNotFoundException ex2) {
		}
	}
	
	/*
	 * ,    .
	 */
	void writeToFile() {
		try {
			ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(System.getProperty("user.home") + "/rebuses" + "/property.prt"));
			os.writeObject(myRebusColor);
			os.writeObject(myBackgroundColor);
			os.writeInt(mySplit.getOrientation());
			os.close();
		} catch (IOException ex) {
		}
	}
	
	/**
	 *  .
	 */
	public RebusForm() {
		super("Rebuses");
		
		myView = new RebusViewPanel(this);
		myResult = new RebusResultPanel(this);
		
		readFromFile();
		
		if(myBackgroundColor != null) {
			myView.setBackground(myBackgroundColor);
			myResult.setBackground(myBackgroundColor);
			mySplit.setBackground(myBackgroundColor);
		}
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				writeToFile();
				System.exit(0);
			}
		});
		
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
		
		// 
		mySplit.setContinuousLayout(true);
		mySplit.setDividerLocation(0.5);
		mySplit.setResizeWeight(0.5);
		mySplit.setOneTouchExpandable(true);
		mySplit.setDividerSize(5);
		mySplit.setAlignmentX(Component.CENTER_ALIGNMENT);
				
		//  
		mySplit.setLeftComponent(new JScrollPane(myView));
		mySplit.setRightComponent(new JScrollPane(myResult));
		
		// 
		myButton = new JButton("Solve");
		myButton.setAlignmentX(Component.CENTER_ALIGNMENT);
		//  
		myButton.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;
			public void actionPerformed(ActionEvent e) {
				if (myButton.isEnabled() && (myRebus != null)) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (myRebus != null) {
								myResult.calculate();
								myResult.setVisible(true);	
								myResult.repaint();
							}
						}
					});
				}
			}
		});
		
		RebusMenu rm = new RebusMenu(this);
		setJMenuBar(rm.getMenu());
		
		// 
		JPanel panel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(rm.getTool());
		getContentPane().add(panel);
		
		//     Content Pane
		getContentPane().add(Box.createVerticalStrut(10));
		getContentPane().add(mySplit);
		getContentPane().add(Box.createVerticalStrut(10));
		
		getContentPane().add(myButton);
		getContentPane().add(Box.createVerticalStrut(10));
		
		setSize(700, 500);
		setVisible(true);
	}
	
	/*
	 * ,   .
	 */
	void setRebusColor(Color color) {
		myRebusColor = color;
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				myResult.repaint();
				myView.repaint();
			}
		});
	}
	
	/*
	 * ,   .
	 */
	void setBackgroundColor(Color color) {
		myBackgroundColor = color;
		myView.setBackground(color);
		myResult.setBackground(color);
		mySplit.setBackground(color);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				myResult.repaint();
				myView.repaint();
			}
		});
	}
	
	/*
	 * ,   .
	 */
	void setOrientation(int orientation) {
		mySplit.setOrientation(orientation);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				RebusForm.this.repaint();
			}
		});
	}
	
	/*
	 * ,   .
	 */
	IRebus getRebus() {
		return myRebus;
	}
	
	/*
	 * ,   .
	 */
	void setDisable() {
		myButton.setEnabled(false);
	}
	
	/*
	 * ,   .
	 */
	void setEnable() {
		myButton.setEnabled(true);
	}
	
	/*
	 * ,   .
	 */
	public void setRebus(IRebus r) {
		myRebus = r;
		
		myView.setVisible(true);
		myResult.setPreferredSize(new Dimension(100, 100));
		myResult.setVisible(false);
		
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				myView.repaint();
				myView.setDimension();
				myView.revalidate();
			}
		});
	}
	
	/**
	 *   .
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		new RebusForm();
	}
}
