package org.olga.rebus.gui;

import java.awt.*;
import java.util.*;

import javax.swing.*;

import org.olga.rebus.calculations.RebusEnumerator;
import org.olga.rebus.calculations.WordsEnumerator;
import org.olga.rebus.calculations.Result.WordResult;
import org.olga.rebus.structure.*;
import org.olga.rebus.structure.Rebus.*;

/**
 * -    .
 * 
 * @author Olga
 *
 */
class RebusResultPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	//,     
	private RebusForm myFrame;
	//    
	private RebusEnumerator myEnumerator;
	//  -       
	private boolean isPainted;
	
	/**
	 *  .
	 * 
	 * @param frame ,     .
	 */
	public RebusResultPanel(RebusForm frame) {
		myFrame = frame;
		isPainted = false;
		this.setPreferredSize(new Dimension(100, 100));
	}
	
	/*
	 * ,   .
	 */
	private void setDimension(int count) {
		IRebus rebus = myFrame.getRebus();
		int weigth = 0;
		int height = 0;
		for (VisibleWord word: rebus.getVisibleWords()) {
			int w = (word.getX() / 2 + word.getElement().toString().length()) * 18;
			int h = word.getY() * 9;
			if (weigth < w) {
				weigth = w;
			}
			if (height < h) {
				height = h;
			}
		}
		this.setPreferredSize(new Dimension(weigth, (height + 15) * count));
	}
	
	/*
	 * ,      
	 *      .
	 */
	void calculate() {
		this.setDimension(1);
		isPainted = false;
		
		Thread calculations = new Thread() {
			public void run() {
				IRebus rebus = myFrame.getRebus();
				if (myEnumerator == null) {
					myEnumerator = new RebusEnumerator(rebus);
				} else {
					myEnumerator.setRebus(rebus);
				}
				myFrame.setCursor(new Cursor(3));
				myFrame.setDisable();
				
				if (rebus.getEnumWords() == null) {
					
					AbstractWord[] words = new AbstractWord[rebus.getVisibleWords().size()];
					
					int k = 0;
					for (VisibleWord word: rebus.getVisibleWords()) {
						words[k++] = word.getElement();
					}
					
					
					WordsEnumerator wordsEnumerator = new WordsEnumerator(words, rebus, myEnumerator);
					wordsEnumerator.findEnums();
										
					/*for (AbstractWord word: rebus.getEnumWords()) {
						System.out.print(word + " ");
					}
					System.out.println();*/
				} else {
					if (!myEnumerator.calculate()) {
						JOptionPane.showMessageDialog(myFrame, "   .", "", 0);
						return;
					} 
				}
				isPainted = true;
				RebusResultPanel.this.setDimension(myEnumerator.getNumberResults());
				
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						RebusResultPanel.this.repaint();
						RebusResultPanel.this.revalidate();
					}
				});
				myFrame.setCursor(new Cursor(0));
				myFrame.setEnable();
			}
		};
		calculations.start();
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		if (isPainted) {
			IRebus rebus = myFrame.getRebus();
	
			Font f = g.getFont();
			g.setColor(myFrame.myRebusColor);
			g.setFont(new Font(f.getName(), Font.BOLD, f.getSize()));
			
			Iterator<LinkedList<WordResult>> it = myEnumerator.getResult();
			
			int maxX = 0;
			int maxY = 0;
			if (!it.hasNext()) {
				String no = " ";
				int i = 0;
				for (char c: no.toCharArray()) {
					int x = 5 + i * 15;
					int y = 15;
					if (maxX < x) {
						maxX = x;
					}
					if (maxY < y) {
						maxY = y;
					}
					g.drawString(String.valueOf(c), x, y);
					i++;
				}
				return;
			}
			int j = 0;
			while (it.hasNext()) {
				Iterator<WordResult> it1 = it.next().iterator();
				Iterator<VisibleWord> it2 = rebus.getVisibleWords().iterator();
				int len = 0;
				while (it1.hasNext()) {
					WordResult wr = it1.next();
					VisibleWord v = it2.next();
					if (v.getY() > len) {
						len = v.getY();
					}
					int i = 0;
					for (char c: String.valueOf(wr.getValue()).toCharArray()) {
						int x = 5 + v.getX() * 15 / 2 + i * 15;
						int y = 15 + v.getY() * 18 / 2 + j * 15;
						if (maxX < x) {
							maxX = x;
						}
						if (maxY < y) {
							maxY = y;
						}
						g.drawString(String.valueOf(c), x, y);
						i++;
					}
					
					
				}
				
				Collection lines = rebus.getLines();
				if (lines != null) {
					for (Object line: lines) {
						VisibleLine l1 = null;
						if (line instanceof VisibleLine) {
							l1 = (VisibleLine) line;
						}
						int x1 = 10 + l1.getX1() * 15 / 2;
						int y1 = 10 + l1.getY1() * 18 / 2 + j * 15;
						int x2 = 10 + l1.getX2() * 15 / 2;
						int y2 = 10 + l1.getY2() * 18 / 2 + j * 15;
						if (maxX < x1) {
							maxX = x1;
						}
						if (maxY < y1) {
							maxY = y1;
						}
						if (maxX < x2) {
							maxX = x2;
						}
						if (maxY < y2) {
							maxY = y2;
						}
						g.drawLine(x1, y1, x2, y2);
					}
				}
				Collection signs = rebus.getSigns();
				
				for (Object c: signs) {
					VisibleSign c1 = null;
					if (c instanceof VisibleSign) {
						c1 = (VisibleSign) c;
					}
					int x = c1.getX() * 15 / 2;
					int y = c1.getY() * 18 / 2 + j * 15;
					if (maxX < x) {
						maxX = x;
					}
					if (maxY < y) {
						maxY = y;
					}
					if (c1.getElement() == '*') {
						g.setFont(new Font(f.getName(), Font.BOLD, f.getSize() + 5));
						g.drawString(String.valueOf(c1.getElement()), 5 + x, 20 + y);
						g.setFont(new Font(f.getName(), Font.BOLD, f.getSize()));
					} else {
						g.drawString(String.valueOf(c1.getElement()), 5 + x, 15 + y);
					}
				}
				j++;
				j += len;
			}
		}
	}
	
	/*
	 *   .
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}
	
	/*
	 *   .
	 */
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		return getPreferredSize().height / 10;
	}

	/*
	 *   .
	 */
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		return getPreferredSize().height;
	}

	/*
	 *   .
	 */
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/*
	 *   .
	 */
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
}
