package org.olga.rebus.gui.creation;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import java.net.URL;

import org.olga.rebus.gui.creation.ModelGraphPaper.Position;
import org.olga.rebus.structure.Rebus;
import org.olga.rebus.structure.Rebus.*;

/**
 * -   .  
 *  .
 * 
 * @author Olga
 *
 */
public class GraphPaper extends JPanel implements Scrollable {
	private static final long serialVersionUID = 1L;
	
	//,   
	static final int CELL_SIZE = 26;
	static final int CELL_COUNT = 24;
	static final Color BACKGROUND_COLOR = new Color(255, 255, 200);
	
	// 
	private ModelGraphPaper myModel = new ModelGraphPaper(CELL_COUNT);
	//  
	private GraphPaperListener myListener = new GraphPaperListener();
	// 
	private GraphPaperRepainter myRepainter = new GraphPaperRepainter();
	
	// 
	private Line myLine = new Line();
	// 
	private Sign mySign = new Sign();
	
	//    
	private JToggleButton mySelectedButton;
	//  
	private SelectedPicture mySelectedPicture;
	// 
	private JToolBar myToolBar;
	
	//  ,     
	private Stack<LinkedList> myLastChange = new Stack<LinkedList>();
	//-  
	HashMap<Position, Cell> myCells = new HashMap<Position, Cell>();
	// 
	LinkedList<VisibleLine> myLines = new LinkedList<VisibleLine>();
	// 
	LinkedList<VisibleSign> mySigns = new LinkedList<VisibleSign>();
	
	//,       
	private Position myCurrentPosition = myModel.getCellPosition(0, 0);

	/**
	 *  .
	 */
	public GraphPaper() {
		super();		
		
		this.addMouseListener(myListener);
		this.addKeyListener(myListener);
		this.addMouseMotionListener(myListener);
		
		//this.setLayout(null);
		this.setFocusable(true);
		this.setFocusCycleRoot(true);
		this.setFocusTraversalKeysEnabled(true);
		this.setFocusTraversalPolicyProvider(true);
		
		myToolBar = this.createToolBar();
		
		this.setBackground(BACKGROUND_COLOR);
		
		setPreferredSize(new Dimension(CELL_SIZE * CELL_COUNT, CELL_SIZE * CELL_COUNT));
		setMaximumSize(new Dimension(CELL_SIZE * CELL_COUNT, CELL_SIZE * CELL_COUNT));
		setMinimumSize(new Dimension(CELL_SIZE * CELL_COUNT, CELL_SIZE * CELL_COUNT));
		setVisible(true);
	}
		
	/**
	 * ,   .
	 * 
	 * @return  .
	 */
	JToolBar getTools() {
		return myToolBar;
	}
	
	/* (non-Javadoc)
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		Font font = g.getFont();
		g.setFont(new Font(font.getName(), Font.BOLD, font.getStyle()));
		int x = myCurrentPosition.myX * CELL_SIZE;
		int y = myCurrentPosition.myY * CELL_SIZE;
		if (mySelectedPicture == null) {
			g.setColor(new Color(Color.GRAY.getRGB()));
			g.drawRect(x + 1, y + 1, CELL_SIZE - 2, CELL_SIZE - 2);
		} else {
			g.setColor(new Color(244, 223, 195));
			int x1 = mySelectedPicture.mySelectX1 * CELL_SIZE;
			int y1 = mySelectedPicture.mySelectY1 * CELL_SIZE;
			int x2 = mySelectedPicture.mySelectX2 * CELL_SIZE;
			int y2 = mySelectedPicture.mySelectY2 * CELL_SIZE;
			g.fillRect(x1, y1, x2 - x1 + CELL_SIZE, y2 - y1 + CELL_SIZE);
		}
		g.setColor(new Color(242, 205, 158));
		for (int i = 0; i < CELL_COUNT; i++) {
			g.drawLine(0, i * CELL_SIZE, CELL_SIZE * CELL_COUNT, i * CELL_SIZE);
			g.drawLine(i * CELL_SIZE, 0, i * CELL_SIZE, CELL_SIZE * CELL_COUNT);
		}
		g.setColor(new Color(Color.BLUE.getRGB()));
		for (VisibleLine line: myLines) {
			int x1 = line.getX1() * CELL_SIZE;
			int x2 = line.getX2() * CELL_SIZE;
			int y1 = line.getY1() * CELL_SIZE;
			int y2 = line.getY2() * CELL_SIZE;
			g.drawLine(x1, y1, x2, y2);
		}
		g.setFont(new Font(font.getName(), Font.BOLD, font.getSize() + 5));
		for (VisibleSign sign: mySigns) {
			
			int xx = (int) (sign.getX() * ((double)CELL_SIZE / 2));
			int yy = (int) (sign.getY() * ((double)CELL_SIZE / 2));
			if (sign.getElement() == '*') {
				g.drawString("" + sign.getElement(), xx - 4, yy + 10);
			} else {
				g.drawString("" + sign.getElement(), xx - 4, yy + 6);
			}
		}
		if ((GraphPaper.this.getCursor().getType() == 1) && (myLine.myIsLinePaint)) {
			g.drawLine(myLine.myLineX1, myLine.myLineY1, myLine.myLineX2, myLine.myLineY2);
		}
		if ((mySign.myCharSign != null) && (!mySign.myOut)) {
			if (mySign.myCharSign == '*') {
				g.drawString(mySign.myCharSign.toString(), mySign.mySignX, mySign.mySignY + 4);
			} else {
				g.drawString(mySign.myCharSign.toString(), mySign.mySignX, mySign.mySignY);
			}
		}
		
		g.setFont(new Font(font.getName(), font.getStyle(), font.getSize()));
		for (Position p: myCells.keySet()) {
			Cell cell = myCells.get(p);
			g.drawString(cell.mySymbol.toString(), cell.myX * CELL_SIZE + 8, cell.myY * CELL_SIZE + 15);
		}
		
		if (mySelectedPicture != null) {
			g.setColor(new Color(Color.RED.getRGB()));
			int x1 = mySelectedPicture.mySelectX1 * CELL_SIZE;
			int y1 = mySelectedPicture.mySelectY1 * CELL_SIZE;
			int x2 = mySelectedPicture.mySelectX2 * CELL_SIZE;
			int y2 = mySelectedPicture.mySelectY2 * CELL_SIZE;
			g.drawRect(x1, y1, x2 - x1 + CELL_SIZE, y2 - y1 + CELL_SIZE);
			if (mySelectedPicture.isChanged) {
				g.drawString("o", (x2 - x1 + CELL_SIZE) / 2 + x1 - 3, y1 + 4);
				g.drawString("o", (x2 - x1 + CELL_SIZE) / 2 + x1 - 3, y2 + CELL_SIZE + 4);
				g.drawString("o", x1 - 3, (y2 - y1 + CELL_SIZE) / 2 + y1 + 4);
				g.drawString("o", x2 + CELL_SIZE - 3, (y2 - y1 + CELL_SIZE) / 2 + y1 + 4);
				g.drawString("o", x1 - 3, y1 + 4);
				g.drawString("o", x1 - 3, y2 + CELL_SIZE + 4);
				g.drawString("o", x2 + CELL_SIZE - 3, y1 + 4);
				g.drawString("o", x2 + CELL_SIZE - 3, y2 + CELL_SIZE + 4);
			}
		}
	}
		
	/* (non-Javadoc)
	 * @see javax.swing.Scrollable#getPreferredScrollableViewportSize()
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return getPreferredSize();
	}

	/* (non-Javadoc)
	 * @see javax.swing.Scrollable#getScrollableUnitIncrement(java.awt.Rectangle, int, int)
	 */
	public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation, int direction) {
		return 0/*CELL_SIZE*/;
	}

	/* (non-Javadoc)
	 * @see javax.swing.Scrollable#getScrollableBlockIncrement(java.awt.Rectangle, int, int)
	 */
	public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation, int direction) {
		return CELL_SIZE * 10;
	}

	/* (non-Javadoc)
	 * @see javax.swing.Scrollable#getScrollableTracksViewportWidth()
	 */
	public boolean getScrollableTracksViewportWidth() {
		return false;
	}

	/* (non-Javadoc)
	 * @see javax.swing.Scrollable#getScrollableTracksViewportHeight()
	 */
	public boolean getScrollableTracksViewportHeight() {
		return false;
	}
	
	/*
	 *  -  .
	 */
	private class GraphPaperListener implements MouseListener, KeyListener, MouseMotionListener {

		public void mouseClicked(MouseEvent e) {
			if (mySelectedPicture != null) {
				if (e.getClickCount() == 2) {
					mySelectedPicture.isChanged = true;
				} else {
					mySelectedPicture.isChanged = false;
				}
				graphRepaint();
			}
		}
		
		public void mousePressed(MouseEvent e) {
			if (GraphPaper.this.getCursor().getType() == 1) {
				int ost1 = e.getX() % CELL_SIZE;
				int ost2 = e.getY() % CELL_SIZE;
				myLine.myLineX1 = e.getX() - ost1;
				myLine.myLineY1 = e.getY() - ost2;
				if (ost1 >= CELL_SIZE / 2) {
					myLine.myLineX1 += CELL_SIZE;
				} 
				if (ost2 >= CELL_SIZE / 2) {
					myLine.myLineY1 += CELL_SIZE;
				} 
				myLine.myIsLinePaint = true;
			}
		}
		public void mouseEntered(MouseEvent e) {
			mySign.myOut = false;
			graphRepaint();	
		}
		public void mouseExited(MouseEvent e) {
			mySign.myOut = true;
			graphRepaint();	
		}
		public void keyTyped(KeyEvent e) {}
		public void keyReleased(KeyEvent e) {}
		
		public void mouseReleased(MouseEvent e) {
			if (mySign.myCharSign != null) {
				int x = (int) (((double)mySign.mySignX + 4) / ((double)CELL_SIZE / 2));
				int y = (int) (((double)mySign.mySignY - 5) / ((double)CELL_SIZE / 2));
				mySigns.add(Rebus.createVisibleSign(mySign.myCharSign, x, y));
				mySelectedButton.setSelected(false);
				myLastChange.push(mySigns);
				mySign.myCharSign = null;
				return;
			}
			if ((GraphPaper.this.getCursor().getType() == 1) && (myLine.myIsLinePaint)) {
				int ost1 = (e.getX() % CELL_SIZE);
				int kx = 0; 
				int ky = 0;
				if (ost1 > CELL_SIZE / 2) {
					kx = 1;
				}
				int ost2 = (e.getY() % CELL_SIZE);
				if (ost2 > CELL_SIZE / 2) {
					ky = 1;
				}
				myLine.myLineX2 = e.getX() - ost1;
				myLine.myLineY2 = e.getY() - ost2;
				int x1 = myLine.myLineX1 / CELL_SIZE;
				int x2 = myLine.myLineX2 / CELL_SIZE + kx;
				int y1 = myLine.myLineY1 / CELL_SIZE;
				int y2 = myLine.myLineY2 / CELL_SIZE + ky;
				myLines.add(Rebus.createVisibleLine(x1, y1, x2, y2));
				myLastChange.push(myLines);
				myLine.myIsLinePaint = false;
				GraphPaper.this.setCursor(new Cursor(0));
				mySelectedButton.setSelected(false);
				graphRepaint();
				return;
			}
			
			int cellX = e.getX() / CELL_SIZE;
			int cellY = e.getY() / CELL_SIZE;
			myCurrentPosition = myModel.getCellPosition(cellX, cellY);
			graphRepaint();			
		}
		
		public void keyPressed(KeyEvent e) {
			//System.out.println(e.getKeyCode());
			if (mySelectedPicture != null) {
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_DOWN) {
				Position next = myModel.getCellPosition(myCurrentPosition.myX, myCurrentPosition.myY + 1);
				if (next != null) {
					myCurrentPosition = next;
					graphRepaint();
				}
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
				Position next = myModel.getCellPosition(myCurrentPosition.myX + 1, myCurrentPosition.myY);
				if (next != null) {
					myCurrentPosition = next;
					graphRepaint();
				}
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_UP) {
				Position next = myModel.getCellPosition(myCurrentPosition.myX, myCurrentPosition.myY - 1);
				if (next != null) {
					myCurrentPosition = next;
					graphRepaint();
				}
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_LEFT) {
				Position next = myModel.getCellPosition(myCurrentPosition.myX - 1, myCurrentPosition.myY);
				if (next != null) {
					myCurrentPosition = next;
					graphRepaint();
				}
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
				Position next = myModel.getCellPosition(myCurrentPosition.myX - 1, myCurrentPosition.myY);
				if (next != null) {
					myCurrentPosition = next;
					if (myCells.containsKey(myCurrentPosition)) {
						myCells.remove(myCurrentPosition);
					}
					graphRepaint();
				}
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_DELETE) {
				if (myCells.containsKey(myCurrentPosition)) {
					myCells.remove(myCurrentPosition);
				}
				graphRepaint();
				return;
			}
			if (e.getKeyCode() == ' ') {
				Position next = myModel.getCellPosition(myCurrentPosition.myX + 1, myCurrentPosition.myY);
				if (next != null) {
					if (myCells.containsKey(myCurrentPosition)) {
						myCells.remove(myCurrentPosition);
					}
					myCurrentPosition = next;
				}
				graphRepaint();
				return;
			}
			
			Cell text = myCells.get(myCurrentPosition);
			boolean flag = text == null;
			if (flag) {
				text = new Cell(myCurrentPosition.myX, myCurrentPosition.myY);
			}
			if (text.setSymbol(e.getKeyChar())) {
				if (flag) {
					myCells.put(myCurrentPosition, text);
				}
				graphRepaint();
				int x = myCurrentPosition.myX;
				int y = myCurrentPosition.myY;
				Position next = myModel.getCellPosition(x + 1, y);
				if (next != null) {
					myCurrentPosition = next;
				}
			}
		}
		
		public void mouseDragged(MouseEvent e) {
			if (mySelectedPicture != null) {
				if (mySelectedPicture.isChanged) {
					int typeCursor = GraphPaper.this.getCursor().getType();
					if (typeCursor == Cursor.N_RESIZE_CURSOR) {
						mySelectedPicture.mySelectY1 = e.getY() / CELL_SIZE;
					} else if (typeCursor == Cursor.S_RESIZE_CURSOR) {
						mySelectedPicture.mySelectY2 = e.getY() / CELL_SIZE;
					} else if (typeCursor == Cursor.W_RESIZE_CURSOR) {
						mySelectedPicture.mySelectX1 = e.getX() / CELL_SIZE;
					} else if (typeCursor == Cursor.E_RESIZE_CURSOR) {
						mySelectedPicture.mySelectX2 = e.getX() / CELL_SIZE;
					} else if (typeCursor == Cursor.NW_RESIZE_CURSOR) {
						mySelectedPicture.mySelectX1 = e.getX() / CELL_SIZE;
						mySelectedPicture.mySelectY1 = e.getY() / CELL_SIZE;
					} else if (typeCursor == Cursor.SW_RESIZE_CURSOR) {
						mySelectedPicture.mySelectX1 = e.getX() / CELL_SIZE;
						mySelectedPicture.mySelectY2 = e.getY() / CELL_SIZE;
					} else if (typeCursor == Cursor.NE_RESIZE_CURSOR) {
						mySelectedPicture.mySelectX2 = e.getX() / CELL_SIZE;
						mySelectedPicture.mySelectY1 = e.getY() / CELL_SIZE;
					} else if (typeCursor == Cursor.SE_RESIZE_CURSOR) {
						mySelectedPicture.mySelectX2 = e.getX() / CELL_SIZE;
						mySelectedPicture.mySelectY2 = e.getY() / CELL_SIZE;
					}
					if (mySelectedPicture.mySelectX2 > CELL_COUNT) {
						mySelectedPicture.mySelectX2 = CELL_COUNT - 1;
					}
					if (mySelectedPicture.mySelectY2 > CELL_COUNT) {
						mySelectedPicture.mySelectY2 = CELL_COUNT - 1;
					}
					if (mySelectedPicture.mySelectX1 < 0) {
						mySelectedPicture.mySelectX1 = 0;
					}
					if (mySelectedPicture.mySelectY1 < 0) {
						mySelectedPicture.mySelectY1 = 0;
					}
					graphRepaint();
					return;
				}
			}
			myLine.myLineX2 = e.getX();
			myLine.myLineY2 = e.getY();
			graphRepaint();
		}
		
		public void mouseMoved(MouseEvent e) {
			if (mySelectedPicture != null) {
				if (mySelectedPicture.isChanged) {
					int x = e.getX();
					int y = e.getY();
					int x1 = mySelectedPicture.mySelectX1 * CELL_SIZE;
					int x2 = mySelectedPicture.mySelectX2 * CELL_SIZE;
					int y1 = mySelectedPicture.mySelectY1 * CELL_SIZE;
					int y2 = mySelectedPicture.mySelectY2 * CELL_SIZE;
					if ((((int)Math.abs(x - ((x2 - x1 + CELL_SIZE) / 2 + x1))) <= 5) && (((int)Math.abs(y - y1)) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.N_RESIZE_CURSOR));
					} else if (((int)Math.abs(x - ((x2 - x1 + CELL_SIZE) / 2 + x1)) <= 5) && (((int)Math.abs(y - (y2 + CELL_SIZE))) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.S_RESIZE_CURSOR));
					} else if(((int)Math.abs(x - x1) <= 5) && (((int)Math.abs(y - ((y2 - y1 + CELL_SIZE) / 2 + y1))) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.W_RESIZE_CURSOR));
					} else if(((int)Math.abs(x - (x2 + CELL_SIZE)) <= 5) && (((int)Math.abs(y - ((y2 - y1 + CELL_SIZE) / 2 + y1))) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
					} else if(((int)Math.abs(x - x1) <= 5) && (((int)Math.abs(y - y1)) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.NW_RESIZE_CURSOR));
					} else if(((int)Math.abs(x - x1) <= 5) && (((int)Math.abs(y - (y2 + CELL_SIZE))) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.SW_RESIZE_CURSOR));
					} else if(((int)Math.abs(x - (x2 + CELL_SIZE)) <= 5) && (((int)Math.abs(y - y1)) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.NE_RESIZE_CURSOR));
					} else if(((int)Math.abs(x - (x2 + CELL_SIZE)) <= 5) && (((int)Math.abs(y - (y2 + CELL_SIZE))) <= 5)) {
						GraphPaper.this.setCursor(new Cursor(Cursor.SE_RESIZE_CURSOR));
					} else {
						GraphPaper.this.setCursor(new Cursor(0));
					}
				}
				return;
			}
			if (mySign.myCharSign != null) {
				mySign.mySignX = (int)((int)(((double)e.getX() - 4) / ((double)CELL_SIZE / 2)) * ((double)CELL_SIZE / 2)) - 4 + CELL_SIZE / 2;
				mySign.mySignY = (int)((int)(((double)e.getY() + 5) / ((double)CELL_SIZE / 2)) * ((double)CELL_SIZE / 2)) + 6;
			}
			graphRepaint();
		}
	}
	
	private ImageIcon createIcon(String name) {
		URL url = GraphPaper.class.getResource("/org/olga/rebus/gui/creation/images/" + name + ".gif");
		return new ImageIcon(url);
	}
	/*
	 * ,   .
	 */
	private JToolBar createToolBar() {
		JToolBar tools = new JToolBar();
		JButton undo = new JButton(createIcon("undo"));
		undo.setFocusable(false);
		undo.setToolTipText("Undo");
		undo.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if ((myLastChange != null) && (myLastChange.size() > 0)) {
					LinkedList list = myLastChange.pop();
					if (list.size() > 0) {
						list.removeLast();
					}
				}
				graphRepaint();	
			}
		});
		JToggleButton line = new JToggleButton(createIcon("line"));
		line.setFocusable(false);
		line.setToolTipText("Create line");
		line.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (mySelectedButton != null) {
					mySelectedButton.setSelected(false);
				}
				unSelectAll();
				mySelectedButton = (JToggleButton) e.getSource();
				GraphPaper.this.setCursor(new Cursor(1));
			}
		});
		JToggleButton plus = new JToggleButton(createIcon("plus"));
		plus.setFocusable(false);
		plus.setToolTipText("Plus");
		plus.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (mySelectedButton != null) {
					mySelectedButton.setSelected(false);
				}
				unSelectAll();
				mySelectedButton = (JToggleButton) e.getSource();
				mySign.myCharSign = '+';
			}
		});
		JToggleButton minus = new JToggleButton(createIcon("minus"));
		minus.setFocusable(false);
		minus.setToolTipText("Minus");
		minus.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (mySelectedButton != null) {
					mySelectedButton.setSelected(false);
				}
				unSelectAll();
				mySelectedButton = (JToggleButton) e.getSource();
				mySign.myCharSign = '-';
			}
		});
		JToggleButton divisionSign = new JToggleButton(createIcon("del"));
		divisionSign.setFocusable(false);
		divisionSign.setToolTipText("Division sign");
		divisionSign.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (mySelectedButton != null) {
					mySelectedButton.setSelected(false);
				}
				unSelectAll();
				mySelectedButton = (JToggleButton) e.getSource();
				mySign.myCharSign = '/';
			}
		});
		JToggleButton obliqueCross = new JToggleButton(createIcon("multip"));
		obliqueCross.setFocusable(false);
		obliqueCross.setToolTipText("Oblique Cross");
		obliqueCross.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (mySelectedButton != null) {
					mySelectedButton.setSelected(false);
				}
				unSelectAll();
				mySelectedButton = (JToggleButton) e.getSource();
				mySign.myCharSign = '*';
			}
		});
		JToggleButton equalsSign = new JToggleButton(createIcon("equal"));
		equalsSign.setFocusable(false);
		equalsSign.setToolTipText("Equals Sign");
		equalsSign.addActionListener(new AbstractAction() {
			private static final long serialVersionUID = 1L;

			public void actionPerformed(ActionEvent e) {
				if (mySelectedButton != null) {
					mySelectedButton.setSelected(false);
				}
				unSelectAll();
				mySelectedButton = (JToggleButton) e.getSource();
				mySign.myCharSign = '=';
			}
		});
		
		tools.add(undo);
		tools.addSeparator();
		tools.add(line);
		tools.addSeparator();
		tools.add(plus);
		tools.add(minus);
		tools.add(divisionSign);
		tools.add(obliqueCross);
		tools.add(equalsSign);
		tools.addSeparator();
		tools.setAlignmentY(Component.TOP_ALIGNMENT);
		
		tools.setFloatable(false);
		tools.setOrientation(JToolBar.VERTICAL);
		tools.setFloatable(false);
		
		return tools;
	}

	/*
	 * ,   .
	 */
	private void unSelectAll() {
		mySign.myCharSign = null;
	}
	
	/*
	 * ,   . 
	 */
	void setSelect() {
		mySelectedPicture = new SelectedPicture();
	}
	
	/*
	 * ,   .
	 */
	void removeSelect() {
		mySelectedPicture = null;
	}
	
	/*
	 *    x    .
	 */
	int getSelectX1() {
		return mySelectedPicture.mySelectX1;
	}
	/*
	 *    x    .
	 */
	int getSelectX2() {
		return mySelectedPicture.mySelectX2;
	}
	/*
	 *    y    .
	 */
	int getSelectY1() {
		return mySelectedPicture.mySelectY1;
	}
	/*
	 *    y    .
	 */
	int getSelectY2() {
		return mySelectedPicture.mySelectY2;
	}
	
	/*
	 * -.
	 */
	static class Cell {
		private static final long serialVersionUID = 1L;
		
		Character mySymbol;
		int myX;
		int myY;
		
		Cell(int x, int y) {
			myX = x;
			myY = y;
		}
	
		boolean setSymbol(char c) {
			if ((Character.isLetterOrDigit(c) || c == '*')) {
				mySymbol = c;
			} else if (c == ' ') {
				mySymbol = null;
			} else {
				return false;
			}
			return true;
		}
	}
	
	/*
	 * - .
	 */
	private class SelectedPicture {
		private static final long serialVersionUID = 1L;

		private boolean isChanged; 
		
		int mySelectX1;
		int mySelectX2;
		int mySelectY1;
		int mySelectY2;
		
		public SelectedPicture() {
			setPictureArea();
			graphRepaint();
		}
		
		/*
		 * ,  .
		 */
		private void setPictureArea() {
			int minX = CELL_SIZE * CELL_COUNT;
			int maxX = 0;
			int minY = CELL_SIZE * CELL_COUNT;
			int maxY = 0;
			for (VisibleLine line: myLines) {
				minX = getMin(line.getX1(), minX);
				minX = getMin(line.getX2(), minX);
				maxX = getMax(line.getX1() - 1, maxX);
				maxX = getMax(line.getX2() - 1, maxX);
				minY = getMin(line.getY1(), minY);
				minY = getMin(line.getY2(), minY);
				maxY = getMax(line.getY1() - 1, maxY);
				maxY = getMax(line.getY2() - 1, maxY);
			}
			for (VisibleSign sign: mySigns) {
				minX = getMin(sign.getX() / 2 - 1, minX);
				maxX = getMax(sign.getX() / 2, maxX);
				minY = getMin(sign.getY() / 2, minY);
				maxY = getMax(sign.getY() / 2, maxY);
			}
			for (Position position: myCells.keySet()) {
				minX = getMin(position.myX, minX);
				maxX = getMax(position.myX, maxX);
				minY = getMin(position.myY, minY);
				maxY = getMax(position.myY, maxY);
			}
			mySelectX1 = minX;
			mySelectX2 = maxX;
			mySelectY1 = minY;
			mySelectY2 = maxY;
		}
		
		private int getMin(int x, int y) {
			if (x < y) {
				return x;
			} else {
				return y;
			}
		}
		
		private int getMax(int x, int y) {
			if (x > y) {
				return x;
			} else {
				return y;
			}
		}
	}
	
	private void graphRepaint() {
		SwingUtilities.invokeLater(myRepainter);	
	}
	
	private class GraphPaperRepainter implements Runnable {
		public void run() {
			GraphPaper.this.repaint();
		}
	}
	
	/*
	 *  -,   
	 *   .
	 */
	private static class Line {
		private boolean myIsLinePaint = false;
		int myLineX1;
		int myLineY1;
		int myLineX2;
		int myLineY2;
	}
	
	/*
	 *  -,  
	 *   .
	 */
	private static class Sign {
		private Character myCharSign = null;
		private boolean myOut = true;
		private int mySignX;
		private int mySignY;
	}
}
