package org.olga.rebus.gui.patterns;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.LinkedList;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;

import org.olga.rebus.gui.RebusForm;
import org.olga.rebus.structure.AbstractRelation;
import org.olga.rebus.structure.AbstractSymbol;
import org.olga.rebus.structure.AbstractWord;
import org.olga.rebus.structure.IRebus;
import org.olga.rebus.structure.LetterSymbol;
import org.olga.rebus.structure.Rebus;
import org.olga.rebus.structure.Word;
import org.olga.rebus.structure.Rebus.VisibleLine;
import org.olga.rebus.structure.Rebus.VisibleSign;
import org.olga.rebus.structure.Rebus.VisibleWord;

public class AdditionPattern extends JFrame {
	private static final long serialVersionUID = 1L;
	
	static final Color BACKGROUND_COLOR = new Color(255, 255, 200);
	static final Color FOREGROUND_COLOR = new Color(149, 49, 18);
	
	private WorkPanel myWorkPanel = new WorkPanel();
	private HashMap<Character, LetterSymbol> myHash = new HashMap<Character, LetterSymbol>();
	
	private RebusForm myFrame;
	
	public AdditionPattern(RebusForm frame) {
		super("Create rebus Addition");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		myFrame = frame;
		
		JLabel title = new JLabel("Input the rebus:");
		title.setOpaque(true);
		title.setBackground(BACKGROUND_COLOR);
		title.setForeground(FOREGROUND_COLOR);
		title.setHorizontalAlignment(JLabel.CENTER);
		Font font = title.getFont();
		title.setFont(new Font(font.getName(), Font.BOLD, font.getSize() + 5));
		
		JButton button = new JButton("Create");
		button.setMaximumSize(new Dimension(100, 20));
		button.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				String op1Str = myWorkPanel.myOperand1.getText().trim();
				String op2Str = myWorkPanel.myOperand2.getText().trim();
				String resultStr = myWorkPanel.myResult.getText().trim();
				AbstractWord op1 = createWord(op1Str);
				AbstractWord op2 = createWord(op2Str);
				AbstractWord result = createWord(resultStr);
				LinkedList<AbstractRelation> relations = new LinkedList<AbstractRelation>();
				relations.add(Rebus.createAdditionRelation(op1, op2, result));
				
				int maxX = op1Str.length();
				if (maxX < op2Str.length()) {
					maxX = op2Str.length();
				}
				if (maxX < resultStr.length()) {
					maxX = resultStr.length();
				}
				int op1X = (maxX - op1Str.length()) * 2;
				int op2X = (maxX - op2Str.length()) * 2;
				int resultX = (maxX - resultStr.length()) * 2;
				int lineX = maxX * 2 - 2;
				if ((op1X == op2X) && (op2X == resultX)) {
					op1X += 2;
					op2X += 2;
					resultX += 2;
					lineX += 2;
				}
				int signX = op1X - 1;
				LinkedList<VisibleWord> visibleWords = new LinkedList<VisibleWord>();
				visibleWords.add(Rebus.createVisibleWord(op1, op1X, 0));
				visibleWords.add(Rebus.createVisibleWord(op2, op2X, 2));
				visibleWords.add(Rebus.createVisibleWord(result, resultX, 4));
				
				LinkedList<VisibleSign> signs = new LinkedList<VisibleSign>();
				signs.add(Rebus.createVisibleSign('+', signX, 1));
				
				LinkedList<VisibleLine> lines = new LinkedList<VisibleLine>();
				lines.add(Rebus.createVisibleLine(0, 3, lineX, 3));
				
				IRebus rebus = new Rebus(relations, visibleWords, signs, lines);
				myFrame.setRebus(rebus);
				AdditionPattern.this.setVisible(false);
			}
		});
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBackground(BACKGROUND_COLOR);
		buttonPanel.add(button);
		
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(title, "North");
		getContentPane().add(myWorkPanel);
		getContentPane().add(buttonPanel, "South");
		
		setSize(300, 200);
		setVisible(true);
	}
	
	private Word createWord(String str) {
		AbstractSymbol[] word = new AbstractSymbol[str.length()];
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if (Character.isDigit(c)) {
				word[i] = Rebus.createNumberSymbol(Integer.parseInt("" + c));
			} else if(c == '*') {
				word[i] = Rebus.createAsteriskSymbol();
			} else {
				if (myHash.containsKey(c)) {
					word[i] = myHash.get(c);
				} else {
					LetterSymbol symbol = Rebus.createLetterSymbol(String.valueOf(c));
					word[i] = symbol;
					myHash.put(c, symbol);
				}
			}
		}
		return Rebus.createWord(word);
	}
	
	private static class WorkPanel extends JPanel {
		private static final long serialVersionUID = 1L;
		
		private JTextField myOperand1 = createInputField();
		private JTextField myOperand2 = createInputField();
		private JTextField myResult = createInputField();
		
		WorkPanel() {
			super();
			setBackground(BACKGROUND_COLOR);
			setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			this.add(Box.createVerticalStrut(15));
			this.add(myOperand1);
			this.add(myOperand2);
			JPanel linePanel = new JPanel();
			linePanel.setBackground(FOREGROUND_COLOR);
			
			linePanel.setMaximumSize(new Dimension(110, 2));
			this.add(Box.createVerticalStrut(5));
			this.add(linePanel);
			this.add(Box.createVerticalStrut(5));
			this.add(myResult);
			this.add(Box.createVerticalStrut(15));
		}
		
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			Font font = g.getFont();
			g.setFont(new Font(font.getName(), Font.BOLD, font.getSize() + 5));
			g.setColor(FOREGROUND_COLOR);
			g.drawString("+", myOperand1.getX() - 10, myOperand1.getY() + 26);
		}
		
		JTextField createInputField() {
			JTextField field = new JTextField("", 10);
			field.setBackground(BACKGROUND_COLOR);
			Font font = field.getFont();
			field.setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
			field.setForeground(FOREGROUND_COLOR);
			field.setBorder(new LineBorder(FOREGROUND_COLOR));
			field.setHorizontalAlignment(JTextField.RIGHT);
			field.setMaximumSize(new Dimension(100, 20));
			return field;
		}
	}

	public static void main(String[] args) {
		//new AdditionPattern();
	}
}
