package org.olga.rebus.io;

import java.io.*;
import java.util.*;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.xml.parsers.*;

import org.olga.rebus.structure.*;
import org.olga.rebus.structure.Rebus.*;
import org.w3c.dom.*;

/**
 *     .
 * 
 * @author Olga
 * @see IRebus
 *
 */
public class RebusReader {

	//-    
	private HashMap<String, AbstractSymbol> mySymbols = new HashMap<String, AbstractSymbol>();
	//-   
	private HashMap<String, AbstractWord> myWords = new HashMap<String, AbstractWord>();
	//  
	private ConcurrentLinkedQueue<Element> myElements = new ConcurrentLinkedQueue<Element>();
	// 
	private LinkedList<Subword> mySubwords = new LinkedList<Subword>();
	
	//,    
	private void clear() {
		mySymbols.clear();
		myWords.clear();
		myElements.clear();
		mySubwords.clear();
	}
	
	//,  
	private Word getSubword(String id, int from, int to) {
		AbstractSymbol[] symbols = new AbstractSymbol[to - from];
		for (int i = 0; i < (to - from); i++) {
			AbstractWord word = myWords.get(id);
			if (word == null) {
				return null;
			}
			symbols[i] = word.getSymbol(i + from);
		}
		Word word = Rebus.createWord(symbols);
		mySubwords.add(Rebus.createSubword(myWords.get(id), word, from, to));
		return word;
	}
	
	//,  
	private Word getWord(String str) {
		AbstractSymbol[] symbols = new AbstractSymbol[str.length()];
		int index = 0;
		for (int k = 0; k < str.length(); k++) {
			String s = String.valueOf(str.charAt(k));
			AbstractSymbol c = null;
			try {
				int n = Integer.parseInt(s);
				if (mySymbols.containsKey(s)) {
					c = mySymbols.get(s);
				} else {
					c = Rebus.createNumberSymbol(n);
					mySymbols.put(s, c);
				}
			} catch (NumberFormatException e) {
				if (s.equals("*")) {
					c = Rebus.createAsteriskSymbol();
				} else {
					if (mySymbols.containsKey(s)) {
						c = mySymbols.get(s);
					} else {
						c = Rebus.createLetterSymbol(s);
						mySymbols.put(s, c);
					}
				}
			}
			
			symbols[index++] = c;
		}
		return Rebus.createWord(symbols);
	}
	
	
	/**
	 * ,    .
	 * 
	 * @param file ,    .
	 * @return .
	 */
	public IRebus read(File file) {
		Rebus rebus = null;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setIgnoringComments(true);
			factory.setValidating(false);
		
			DocumentBuilder parser = factory.newDocumentBuilder();
			Document document = parser.parse(file);
				
			NodeList words = document.getElementsByTagName("word");
			NodeList relations = document.getElementsByTagName("relation");
			LinkedList<AbstractRelation> rebusRelations = new LinkedList<AbstractRelation>();
			LinkedList<VisibleWord> views = new LinkedList<VisibleWord>();
			LinkedList<AbstractWord> enums = new LinkedList<AbstractWord>();
		
			for (int i = 0; i < words.getLength(); i++) {
				AbstractWord word;
				Element element = (Element) words.item(i);
				String id = element.getAttribute("id");
				String type = element.getAttribute("type");
				if (type.equals("DummyWord")) {
					word = Rebus.createDummyWord();
				} else {
					if (!element.getAttribute("substring").equals("")) {
						String idFrom = element.getAttribute("substring");
						int from = Integer.parseInt(element.getAttribute("from"));
						int to = Integer.parseInt(element.getAttribute("to"));
						word = getSubword(idFrom, from, to);
						if (word == null) {
							myElements.add(element);
							continue;
						}
					} else {
						word = getWord(element.getAttribute("name"));
					}
					if (element.getAttribute("show").equals("true")) {
						int x = Integer.parseInt(element.getAttribute("positionX"));
						int y = Integer.parseInt(element.getAttribute("positionY"));
						views.add(Rebus.createVisibleWord(word, x, y));
					}
					if (element.getAttribute("enum").equals("true")) {
						enums.add(word);
					}
				}
				myWords.put(id, word);
			}
		
			while (!myElements.isEmpty()) {
				AbstractWord word;
				Element element = myElements.poll();
				String idFrom = element.getAttribute("substring");
				int from = Integer.parseInt(element.getAttribute("from"));
				int to = Integer.parseInt(element.getAttribute("to"));
				word = getSubword(idFrom, from, to);
				if (word == null) {
					myElements.add(element);
					continue;
				}
				if (element.getAttribute("show").equals("true")) {
					int x = Integer.parseInt(element.getAttribute("positionX"));
					int y = Integer.parseInt(element.getAttribute("positionY"));
					views.add(Rebus.createVisibleWord(word, x, y));
				}
				if (element.getAttribute("enum").equals("true")) {
					enums.add(word);
				}
				myWords.put(element.getAttribute("id"), word);
			}
			
			for (int i = 0; i < relations.getLength(); i++) {
				AbstractRelation relation = null;
				Element element = (Element) relations.item(i);
				String idOp1 = element.getAttribute("operand1");
				String idOp2 = element.getAttribute("operand2");
				String idResult = element.getAttribute("result");
				String type = element.getAttribute("type");
				AbstractWord op1 = myWords.get(idOp1);
				AbstractWord op2 = myWords.get(idOp2);
				AbstractWord result = null;
				if (!idResult.equals("")) {
					result = myWords.get(idResult);
				}
				if (type.equals("EqualityRelation")) {
					relation = Rebus.createEqualityRelation(op1, op2);
				} else if (type.equals("AdditionRelation")) {
					relation = Rebus.createAdditionRelation(op1, op2, result);
				} else if (type.equals("MultiplyRelation")) {
					relation = Rebus.createMultiplyRelation(op1, op2, result);
				}
				rebusRelations.add(relation);
			}
		
			LinkedList<VisibleSign> signs = new LinkedList<VisibleSign>();
			LinkedList<VisibleLine> lines = new LinkedList<VisibleLine>();
			NodeList list = document.getElementsByTagName("show");
			for (int i = 0; i < list.getLength(); i++) {
				Element element = (Element) list.item(i);
				if (element.getAttribute("type").equals("sign")) {
					signs.add(Rebus.createVisibleSign(element.getAttribute("name").charAt(0), 
							Integer.parseInt(element.getAttribute("positionX")), 
							Integer.parseInt(element.getAttribute("positionY"))));
				} else {
					lines.add(Rebus.createVisibleLine( 
							Integer.parseInt(element.getAttribute("positionX1")), 
							Integer.parseInt(element.getAttribute("positionY1")),
							Integer.parseInt(element.getAttribute("positionX2")), 
							Integer.parseInt(element.getAttribute("positionY2"))));
				}
			}
		
			rebus = new Rebus(rebusRelations, views, signs, lines);
			rebus.setEnumWords(enums);
			rebus.setSubwords(mySubwords);
		} catch (IOException e) {
			System.out.println(e.getMessage());
			return null;
		} catch (ParserConfigurationException e1) {
			System.out.println(e1.getMessage());
			return null;
		} catch (org.xml.sax.SAXException e2) {
			System.out.println(e2.getMessage());
			return null;
		}
		clear();
		return rebus;
	}
}
