package org.olga.rebus.structure;

import java.util.*;

/**
 *  .
 * 
 * @author Olga
 *
 */
public class Rebus implements IRebus {
	// 
	private LinkedList<AbstractRelation> myRelations;
	//  
	private LinkedList<VisibleWord> myWords;
	// 
	private LinkedList<VisibleSign> mySigns;
	// 
	private LinkedList<VisibleLine> myLines;
	//  
	private LinkedList<AbstractWord> myEnumWords;
	// 
	private LinkedList<Subword> mySubwords;
	
	/**
	 *  .
	 * 
	 * @param relations -  .
	 */
	public Rebus(LinkedList<AbstractRelation> relations, LinkedList<VisibleWord> words, LinkedList<VisibleSign> signs, LinkedList<VisibleLine> lines) {
		myRelations = relations;
		myWords = words;
		mySigns = signs;
		myLines = lines;
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#getRelations()
	 */
	public Collection<AbstractRelation> getRelations() {
		return Collections.unmodifiableCollection(myRelations);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for(AbstractRelation relation: myRelations) {
			sb.append(relation.toString() + "\n");
		}
		return sb.toString();
	}
	
	/**
	 * ,     .
	 * 
	 * @return  .
	 */
	public int getLength() {
		return myRelations.size();
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#getVisibleWords()
	 */
	public Collection<VisibleWord> getVisibleWords() {
		return Collections.unmodifiableCollection(myWords);
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#getSigns()
	 */
	public Collection<VisibleSign> getSigns() {
		return Collections.unmodifiableCollection(mySigns);
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#getLines()
	 */
	public Collection<VisibleLine> getLines() {
		if (myLines != null) {
			return Collections.unmodifiableCollection(myLines);
		} else {
			return null;
		}
	}
	
	/**
	 * ,  -.
	 * 
	 * @return -.
	 */
	public static AsteriskSymbol createAsteriskSymbol() {
		return new AsteriskSymbol();
	}
	
	/**
	 * ,  -.
	 * 
	 * @param name .
	 * @return -.
	 */
	public static LetterSymbol createLetterSymbol(String name) {
		return new LetterSymbol(name);
	}
	
	/**
	 * ,  -.
	 * 
	 * @param value  .
	 * @return -.
	 */
	public static NumberSymbol createNumberSymbol (int value) {
		return new NumberSymbol(value);
	}
	
	/**
	 * ,  .
	 * 
	 * @param word   .
	 * @return .
	 */
	public static Word createWord(AbstractSymbol[] word) {
		return new Word(word);
	}
	
	/**
	 * ,   .
	 * 
	 * @return  .
	 */
	public static DummyWord createDummyWord() {
		return new DummyWord();
	}
	
	/**
	 * ,   .
	 * 
	 * @param op1  .
	 * @param op2  .
	 * @param result  .
	 * @return  .
	 */
	public static MultiplyRelation createMultiplyRelation(AbstractWord op1, AbstractWord op2, AbstractWord result) {
		return new MultiplyRelation(op1, op2, result);
	}
	
	/**
	 * ,   .
	 * 
	 * @param op1  .
	 * @param op2  .
	 * @param result  .
	 * @return  .
	 */
	public static AdditionRelation createAdditionRelation(AbstractWord op1, AbstractWord op2, AbstractWord result) {
		return new AdditionRelation(op1, op2, result);
	}
	
	/**
	 * ,   .
	 * 
	 * @param op1  .
	 * @param op2  .
	 * @return  .
	 */
	public static EqualityRelation createEqualityRelation(AbstractWord op1, AbstractWord op2) {
		return new EqualityRelation(op1, op2);
	}
	
	/**
	 * ,   .
	 * 
	 * @param word .
	 * @param x  x.
	 * @param y  y.
	 * @return  .
	 */
	public static VisibleWord createVisibleWord(AbstractWord word, int x, int y) {
		return new VisibleWord(word, x, y);
	}
	
	/**
	 * ,  .
	 * 
	 * @param c .
	 * @param x  x.
	 * @param y  y.
	 * @return .
	 */
	public static VisibleSign createVisibleSign(char c, int x, int y) {
		return new VisibleSign(c, x, y);
	}
	
	/**
	 * ,  .
	 * 
	 * @param x1  x  .
	 * @param y1  y  .
	 * @param x2  x  .
	 * @param y2  y  .
	 * @return .
	 */
	public static VisibleLine createVisibleLine(int x1, int y1, int x2, int y2) {
		return new VisibleLine(x1, y1, x2, y2);
	}
	
	/**
	 * ,  .
	 * 
	 * @param w1 ,   .
	 * @param w2 ,   .
	 * @param from   .
	 * @param to   .
	 * @return .
	 */
	public static Subword createSubword(AbstractWord w1, AbstractWord w2, int from, int to) {
		return new Subword(w1, w2, from, to);
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#setEnumWords(java.util.LinkedList)
	 */
	public void setEnumWords(LinkedList<AbstractWord> enumWords) {
		myEnumWords = enumWords;
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#getEnumWords()
	 */
	public Collection<AbstractWord> getEnumWords() {
		if (myEnumWords == null) {
			return null;
		}
		return Collections.unmodifiableCollection(myEnumWords);
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#getSubwords()
	 */
	public Collection<Subword> getSubwords() {
		if (mySubwords != null) {
			return Collections.unmodifiableCollection(mySubwords);
		} else {
			return null;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.olga.rebus.structure.IRebus#setSubwords(java.util.LinkedList)
	 */
	public void setSubwords(LinkedList<Subword> sub) {
		mySubwords = sub;
	}
	
	/**
	 *  .
	 * 
	 * @author Olga
	 *
	 */
	public static class Subword {
		//  
		private AbstractWord myFromWord;
		// 
		private AbstractWord myToWord;
		//  
		private int myFrom;
		//  
		private int myTo;
		
		/**
		 *  .
		 * 
		 * @param word1   .
		 * @param word2  .
		 * @param from   .
		 * @param to   .
		 */
		Subword(AbstractWord word1, AbstractWord word2, int from, int to) {
			myFromWord = word1;
			myToWord = word2;
			myFrom = from;
			myTo = to;
		}
		
		/**
		 * ,      .
		 * 
		 * @return ,    .
		 */
		public AbstractWord getFromWord() {
			return myFromWord;
		}
		
		/**
		 * ,  ,   .
		 *  
		 * @return ,  .
		 */
		public AbstractWord getToWord() {
			return myToWord;
		}
		
		/**
		 * , ,      
		 *  .
		 * 
		 * @return  ,    .
		 */
		public int getFrom() {
			return myFrom;
		}
		
		/**
		 * , ,     
		 *  .
		 * 
		 * @return  ,      .
		 */
		public int getTo() {
			return myTo;
		}
		
		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		public String toString() {
			StringBuffer sb = new StringBuffer();
			sb.append(myFromWord.toString());
			sb.append(" ");
			sb.append(myToWord.toString());
			sb.append(" ");
			sb.append(myFrom);
			sb.append(" ");
			sb.append(myTo);
			return sb.toString();
		}
	}
	
	/**
	 *   .
	 * 
	 * @author Olga
	 *
	 */
	public static class VisibleWord {
		//
		private AbstractWord myElement;
		// x
		private int myX;
		// y
		private int myY;
		
		/**
		 *  .
		 * 
		 * @param element .
		 * @param x  x.
		 * @param y  y.
		 */
		VisibleWord(AbstractWord element, int x, int y) {
			myElement = element;
			myX = x;
			myY = y;
		}
		
		/**
		 * ,   x.
		 * 
		 * @return  x.
		 */
		public int getX() {
			return myX;
		}
		
		/**
		 * ,   y.
		 * 
		 * @return  y.
		 */
		public int getY() {
			return myY;
		}
		
		/**
		 * ,  .
		 * 
		 * @return .
		 */
		public AbstractWord getElement() {
			return myElement;
		}
	}
	
	/**
	 *  .
	 * 
	 * @author Olga
	 *
	 */
	public static class VisibleSign {
		// 
		private char myElement;
		// x
		private int myX;
		// y
		private int myY;
		
		/**
		 *  .
		 * 
		 * @param element  .
		 * @param x  x.
		 * @param y  y.
		 */
		VisibleSign(char element, int x, int y) {
			myElement = element;
			myX = x;
			myY = y;
		}
		
		/**
		 * ,   x.
		 * 
		 * @return  x.
		 */
		public int getX() {
			return myX;
		}
		
		/**
		 * ,   y.
		 * 
		 * @return  y.
		 */
		public int getY() {
			return myY;
		}
		
		/**
		 * ,   .
		 * 
		 * @return    .
		 */
		public char getElement() {
			return myElement;
		}
	}
	
	/**
	 *  .
	 * 
	 * @author Olga
	 *
	 */
	public static class VisibleLine {
		// x  
		private int myX1;
		// y  
		private int myY1;
		// x  
		private int myX2;
		// y  
		private int myY2;
		
		/**
		 *  .
		 * 
		 * @param x1  x  .
		 * @param y1  y  .
		 * @param x2  x  .
		 * @param y2  y  .
		 */
		VisibleLine(int x1, int y1, int x2, int y2) {
			myX1 = x1;
			myY1 = y1;
			myX2 = x2;
			myY2 = y2;
		}
		
		/**
		 * ,   x  .
		 * 
		 * @return  x  .
		 */
		public int getX1() {
			return myX1;
		}
		
		/**
		 * ,   y  .
		 * 
		 * @return  y  .
		 */
		public int getY1() {
			return myY1;
		}
		
		/**
		 * ,   x  .
		 * 
		 * @return  x  .
		 */
		public int getX2() {
			return myX2;
		}
		
		/**
		 * ,   y  .
		 * 
		 * @return  y  .
		 */
		public int getY2() {
			return myY2;
		}
	}
}
